#!/usr/bin/env python
"""

source: https://github.com/carljm/django-model-utils/blob/f8a7c50/runtests.py

"""
import os
import sys

from django.conf import settings
import django


ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

LOGGING_CONFIG = {
    'version': 1,
    # 'disable_existing_loggers': True,
    'disable_existing_loggers': False,
    'formatters': {
        'custom': {
            # available LogRecord attributes:
            # http://docs.python.org/library/logging.html#logrecord-attributes
            'format': ('%(levelname)s [%(asctime)s] %(name)s | '
                       '%(funcName)s:%(lineno)d | %(message)s\n')
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'custom'
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', ],
            'level': 'WARNING',
            'propagate': False,
        },
        'django.request': {
            'handlers': ['console', ],
            'level': 'WARNING',
            'propagate': False,
        },
        'lu_dj_utils': {
            'handlers': ['console', ],
            'level': 'WARNING',
            'propagate': False,
        },
    }
}

DEFAULT_SETTINGS = dict(
    INSTALLED_APPS=(
        'django.contrib.sites',  # used by `lu_dj_utils.urls`
        'lu_dj_utils',
    ),
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            }
        },
    TEMPLATE_DIRS=(
        os.path.join(ROOT_DIR, 'tests', 'templates'),
    ),
    LOGGING=LOGGING_CONFIG,
    SILENCED_SYSTEM_CHECKS=[
        '1_7.W001',  # MIDDLEWARE_CLASSES is not set
    ],
)


def runtests():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)

    # Compatibility with Django 1.7's stricter initialization
    if hasattr(django, 'setup'):
        django.setup()

    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)

    try:
        from django.test.runner import DiscoverRunner
        runner_class = DiscoverRunner
        test_args = ['tests']
    except ImportError:
        from django.test.simple import DjangoTestSuiteRunner
        runner_class = DjangoTestSuiteRunner
        test_args = ['tests']

    failures = runner_class(
        verbosity=1, interactive=True, failfast=False).run_tests(test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests()



# coding: utf-8
"""Some useful Django template context processors.

"""
from __future__ import absolute_import, print_function, unicode_literals

from django.conf import settings


def track_visit(request):
    """Whether the current visit should be tracked by analytics software/services.

    .. warning:
       The setting ``TESTING`` must be defined.

    A visit should not be tracked if the website is running in debug mode, in test or
    if the user is a staff member.

    Based in ``lookup_www0/lookup/context_processors.py/do_register_visit``.

    """
    do_track_visit = True
    if settings.DEBUG or settings.TESTING:
        do_track_visit = False
    if hasattr(request, 'user'):
        if request.user.is_staff:
            do_track_visit = False
    return {'track_visit': do_track_visit}



# coding: utf-8
"""Source: lookup_www.common.tests.test_utils.mail (with some edition).

"""
from __future__ import absolute_import, print_function, unicode_literals

from mock import MagicMock, Mock, NonCallableMock, NonCallableMagicMock, patch

from django.conf import settings
from django.core import mail as django_mail
from django.test import SimpleTestCase, TestCase


class PlainEmail(SimpleTestCase):

    def setUp(self):
        NCMock = NonCallableMock

        self.subject = NCMock(spec=str)
        self.body = NCMock(spec=str)
        self.from_email = NCMock(spec=str)

        # self.to = NonCallableMagicMock(spec=list)
        # self.to.__iter__.return_value = ['to0', 'to1', 'to2']
        # self.cc = NonCallableMagicMock(spec=list)
        # self.cc.__iter__.return_value = ['cc0', 'cc1', 'cc2']
        # self.bcc = NonCallableMagicMock(spec=list)
        # self.bcc.__iter__.return_value = ['bcc0', 'bcc1', 'bcc2']

        self.to = ['to0', 'to1', 'to2']
        self.cc = ['cc0', 'cc1', 'cc2']
        self.bcc = ['bcc0', 'bcc1', 'bcc2']

        self.reply_to = NCMock(spec=str)
        self.fail_silently = NCMock(spec=bool)
        self.auth_user = NCMock(spec=str)
        self.auth_password = NCMock(spec=str)
        self.connection = NCMock()
        self.attachments = NCMock(spec=list)
        self.headers = NonCallableMagicMock(spec=dict)

    def test_create_message(self):
        from lu_dj_utils.email import PlainEmail

        msg = PlainEmail().create_message()
        self.assertIsInstance(msg, PlainEmail.django_mail_class)

        msg = PlainEmail(
            subject=self.subject, body=self.body, from_email=self.from_email,
            to=self.to, bcc=self.bcc, connection=self.connection,
            attachments=self.attachments, headers=self.headers, cc=self.cc
        ).create_message()
        self.assertIsInstance(msg, PlainEmail.django_mail_class)
        self.basic_msg_assertions(msg)

    def test_real_send(self):
        from lu_dj_utils.email import PlainEmail

        _reset_test_outbox()
        self.assertEqual(len(django_mail.outbox), 0)

        subject = 'hello!'
        body = 'my email body'
        to = ['a@a.cl', 'x@xx.com']

        result = PlainEmail(subject=subject, body=body).send()
        self.assertEqual(result, 0)

        result = PlainEmail(subject=subject, body=body, to=to).send()
        self.real_send_assertions(to, body, result)

    def basic_msg_assertions(self, msg):
        self.assertEqual(self.subject, msg.subject)
        self.assertEqual(self.body, msg.body)
        self.assertEqual(self.from_email, msg.from_email)
        self.assertEqual(self.to, msg.to)
        self.assertEqual(self.cc, msg.cc)
        self.assertEqual(self.bcc, msg.bcc)
        self.assertEqual(self.connection, msg.connection)
        self.assertEqual(self.attachments, msg.attachments)
        self.assertDictContainsSubset(self.headers, msg.extra_headers)

    def real_send_assertions(self, to, body, result):
        outbox = django_mail.outbox
        self.assertEqual(result, 1)
        self.assertEqual(len(outbox), 1)
        self.assertEqual(outbox[0].to, to)
        self.assertIn(body, outbox[0].body)


class HTMLEmail(PlainEmail):

    def setUp(self):
        super(HTMLEmail, self).setUp()
        self.html_body = NonCallableMock(spec=str)

    def test_create_message(self):
        from lu_dj_utils.email import HTMLEmail

        msg = HTMLEmail(self.html_body).create_message()
        self.assertIsInstance(msg, HTMLEmail.django_mail_class)

        msg = HTMLEmail(
            self.html_body,
            subject=self.subject, body=self.body, from_email=self.from_email,
            to=self.to, bcc=self.bcc, connection=self.connection,
            attachments=self.attachments, headers=self.headers, cc=self.cc
        ).create_message()
        self.assertIsInstance(msg, HTMLEmail.django_mail_class)
        self.basic_msg_assertions(msg)
        self.assert_HTML_alternative(msg, self.html_body)

    def test_real_send(self):
        from lu_dj_utils.email import HTMLEmail, render_to_string

        _reset_test_outbox()
        self.assertEqual(len(django_mail.outbox), 0)

        subject = 'hello!'
        body = 'my email body'
        html_body_template_name = 'email1_body.html'
        to = ['a@a.cl', 'x@xx.com']

        html_body = render_to_string(html_body_template_name)

        result = HTMLEmail(
            html_body, subject=subject, body=body).send()
        self.assertEqual(result, 0)

        result = HTMLEmail(
            html_body, subject=subject, body=body, to=to).send()
        self.real_send_assertions(to, body, result)
        self.assert_HTML_alternative(django_mail.outbox[0], html_body)

    def assert_HTML_alternative(self, message, html_body):
        html_alternative = message.alternatives[0]
        self.assertEqual(html_alternative[0], html_body)
        self.assertEqual(html_alternative[1], 'text/html')


# noinspection PyUnresolvedReferences
class SendMail2(SimpleTestCase):

    def setUp(self):
        NCMock = NonCallableMock

        self.subject = NCMock(spec=str)
        self.body = NCMock(spec=str)
        self.from_email = NCMock(spec=str)
        self.to = NonCallableMagicMock(spec=list)
        self.cc = NonCallableMagicMock(spec=list)
        self.bcc = NonCallableMagicMock(spec=list)
        self.reply_to = NCMock(spec=str)
        self.fail_silently = NCMock(spec=bool)
        self.auth_user = NCMock(spec=str)
        self.auth_password = NCMock(spec=str)
        self.connection = NCMock()
        self.attachments = NCMock(spec=list)
        self.headers = NonCallableMagicMock(spec=dict)

    def test_simplest_call(self):
        from lu_dj_utils.email import send_mail2

        self.assertEqual(send_mail2(), 0)

    def test_full_call(self):
        """Test the function using all the parameters with mock objects."""
        from lu_dj_utils.email import send_mail2

        (subject, body, from_email, to, cc, bcc, reply_to, fail_silently,
            auth_user, auth_password, connection, attachments, headers) = \
        (self.subject, self.body, self.from_email, self.to, self.cc, self.bcc,
            self.reply_to, self.fail_silently, self.auth_user,
            self.auth_password, self.connection, self.attachments,
            self.headers)
        send_return_value = NonCallableMock()

        with patch.object(django_mail.EmailMessage, '__init__', return_value=None) as mock_init:
            with patch.object(django_mail.EmailMessage, 'send') as mock_send:
                mock_send.return_value = send_return_value

                rvalue = send_mail2(
                    subject, body, from_email, to, cc,
                    bcc, reply_to, fail_silently, auth_user, auth_password,
                    connection, attachments, headers)

                # check EmailMessage object construction
                mock_init.assert_called_once_with(subject, body, from_email,
                    to, bcc, connection, attachments, headers, cc)
                # check the returned value is the result of EmailMessage's send method
                self.assertEqual(rvalue, send_return_value)

    def test_connection(self):
        """Check which ``connection`` object is used to create the
        :class:`EmailMessage` object.

        If ``connection`` is passed, that one is used. If not, the result of
        :func:`django.core.mail.get_connection` is used.

        """
        from lu_dj_utils.email import send_mail2

        # Case A: if 'connection' is passed, that is used
        with patch.object(django_mail.EmailMessage, '__init__', return_value=None) as mock_init:
            with patch.object(django_mail.EmailMessage, 'send'):
                send_mail2(connection=self.connection)

                mock_init.assert_called_once_with(
                    '', '', None, None, None,
                    self.connection, None, None, None)

        # Case B: else, the result of 'get_connection' is used
        # (also, the arguments of that call are checked)
        with patch('django.core.mail.get_connection') as mock_get_connection:
            mock_connection = Mock(name='connection')
            mock_get_connection.return_value = mock_connection

            with patch.object(django_mail.EmailMessage, '__init__', return_value=None) as mock_init:
                with patch.object(django_mail.EmailMessage, 'send'):
                    send_mail2()

                    mock_init.assert_called_once_with(
                        '', '', None, None, None, mock_connection, None, None, None)
                    mock_get_connection.assert_called_once_with(
                        username=None, password=None, fail_silently=False)

    def test_reply_to(self):
        from lu_dj_utils.email import send_mail2

        mock_conn = Mock(name='connection')

        with patch('django.core.mail.get_connection') as mock_get_connection:
            mock_get_connection.return_value = mock_conn

            with patch.object(django_mail.EmailMessage, '__init__', return_value=None) as mock_init:
                with patch.object(django_mail.EmailMessage, 'send'):

                    send_mail2(headers=self.headers)
                    mock_init.assert_called_once_with(
                        '', '', None, None, None,
                        mock_conn, None, self.headers, None)
                    self.assertFalse(self.headers.__setitem__.called)

                    mock_init.reset_mock()
                    self.headers.reset_mock()

                    send_mail2(reply_to=self.reply_to, headers=self.headers)
                    mock_init.assert_called_once_with(
                        '', '', None, None, None,
                        mock_conn, None, self.headers, None)
                    self.headers.__setitem__.assert_called_once_with(
                        'Reply-To', self.reply_to)

                    mock_init.reset_mock()
                    self.headers.reset_mock()

                    send_mail2(reply_to=self.reply_to)
                    mock_init.assert_called_once_with('', '', None, None, None,
                        mock_conn, None, {'Reply-To': self.reply_to}, None)
                    self.assertFalse(self.headers.__setitem__.called)

    def test_real_call(self):
        from lu_dj_utils.email import send_mail2

        _reset_test_outbox()
        outbox = django_mail.outbox

        subject = 'hello!'
        body = 'my email body'
        to = ['a@a.cl', 'x@xx.com']

        self.assertEqual(len(outbox), 0)
        send_mail2(subject=subject, body=body, to=to)
        self.assertEqual(len(outbox), 1)
        self.assertEqual(outbox[0].to, to)
        self.assertIn(body, outbox[0].body)

    def test_invalid_addresses(self):
        from lu_dj_utils.email import send_mail2

        _reset_test_outbox()
        outbox = django_mail.outbox

        subject = 'hello!'
        body = 'my email body'
        invalid_addresses = ['', ' ', 'x', 'x@x']
        not_str_addresses = [None, -15]

        # empty 'to' list: sends nothing
        send_mail2(subject=subject, body=body, to=[])
        self.assertEqual(len(outbox), 0)

        # Even though the addresses are invalid, the messages to them ARE SENT
        for address in invalid_addresses:
            temp = len(outbox)
            send_mail2(subject=subject, body=body, to=[address])
            self.assertEqual(len(outbox), temp + 1)

        # if address contains a newline, BadHeaderError(ValueError) is raised
        with self.assertRaises(ValueError):
            send_mail2(subject=subject, body=body, to=['\n'])

        # if address is not a string, ValueError is raised
        for address in not_str_addresses:
            with self.assertRaises(ValueError):
                send_mail2(subject=subject, body=body, to=[address])


class SendTemplatedMailTest(TestCase):

    def setUp(self):
        self.valid_address = 'pedro@perez.cl'
        self.invalid_address = 'pedro@perez'

        self.username = NonCallableMock()
        self.context = dict(nombre=self.username)

        # these templates must exist
        self.subject_name = 'email1_subject.txt'
        self.body_name = 'email1_body.txt'
        self.html_body_name = 'email1_body.html'

    def test_enviar_correo_with_valid_address(self):
        address = self.valid_address
        self.assertTrue(self._send_default(address))

    def test_enviar_correo_with_invalid_address(self):
        """It **will** send the message, even if the address is invalid."""
        address = self.invalid_address
        self.assertTrue(self._send_default(address))

    def test_enviar_correo(self):
        _reset_test_outbox()
        outbox = django_mail.outbox

        address = self.valid_address

        # there is no emails before sending the email
        self.assertEqual(len(outbox), 0)

        # we trigger the action that send the email
        self._send_default(address)

        # email actually sent?
        self.assertEqual(len(outbox), 1)

        # the email is sent to the right destination
        self.assertEqual(outbox[0].to, [address])
        self.assertEqual(outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)

    def test_html(self):
        from lu_dj_utils.email import render_to_string, send_templated_mail

        _reset_test_outbox()
        outbox = django_mail.outbox
        self.assertEqual(len(outbox), 0)

        email_address = self.valid_address

        result = send_templated_mail(
            self.context, self.subject_name, self.body_name,
            self.html_body_name, to=[email_address])
        self.assertTrue(result)
        self.assertEqual(len(outbox), 1)

        message = outbox[0]
        html_body = render_to_string(self.html_body_name, self.context)
        html_alternative = message.alternatives[0]
        self.assertEqual(html_alternative[0], html_body)
        self.assertEqual(html_alternative[1], 'text/html')

    def _send_default(self, email_address):
        from lu_dj_utils.email import send_templated_mail

        return send_templated_mail(
            self.context, self.subject_name, self.body_name, to=[email_address])


class Functions(SimpleTestCase):

    # noinspection PyTypeChecker
    def test_set_header_reply_to(self):
        from lu_dj_utils.email import set_header_reply_to

        func = set_header_reply_to

        email1 = NonCallableMock(spec=str)
        email2 = NonCallableMock(spec=str)

        self.assertEqual(func(None, email1), {'Reply-To': email1})
        self.assertEqual(func({}, email1), {'Reply-To': email1})
        self.assertEqual(func({'Reply-To': email1}, email2), {'Reply-To': email2})
        self.assertIsNone(func(None, None))
        self.assertEqual(func({}, None), {})


def _reset_test_outbox():
    """Empty the outbox manually.

    Apparently there is a bug in TestCase because the test runner should clear
    the contents of the test email outbox at the start of each test case,
    and it is not doing that.

    https://docs.djangoproject.com/en/dev/topics/testing/overview/#emptying-the-test-outbox

    """
    django_mail.outbox = []



# coding: utf-8
"""Source: lookup_www.common.utils.tests.test_utils_test.

"""
from __future__ import absolute_import, print_function, unicode_literals

from django.test import SimpleTestCase, TestCase
from django.utils import six


class FunctionsTest(TestCase):  # TODO: change to SimpleTestCase or unittest.TestCase?

    def test_create_request(self):
        from django.core.handlers.wsgi import WSGIRequest
        from lu_dj_utils.test import create_request

        req = create_request('my-path', 'get')
        self.assertIsInstance(req, WSGIRequest)
        self.assertEqual(req.META['PATH_INFO'], 'my-path')
        self.assertEqual(req.META['REQUEST_METHOD'], 'GET')

        data = {'x': 'y', 'a': 1}
        req = create_request('something', 'post', data=data)
        self.assertIsInstance(req, WSGIRequest)
        self.assertEqual(req.META['PATH_INFO'], 'something')
        self.assertEqual(req.META['REQUEST_METHOD'], 'POST')
        six.assertCountEqual(  # in Python2 = assertItemsEqual
            self,
            req.POST.items(),
            [('x', 'y'), ('a', '1')])



# coding: utf-8
"""Source: lookup_www.common.views (but just a subset).

"""
from __future__ import absolute_import, print_function, unicode_literals

import logging


logger = logging.getLogger(__name__)


class SetTestCookieMixin(object):

    """Override :meth:`get` to set test cookie, creating a session in advance.

    This lets us track a visitor by effectively knowing its session key before
    he/she performs any "action".

    """

    def get(self, request, *args, **kwargs):
        """Override to set a cookie (create a session) to track user."""
        try:
            request.session.set_test_cookie()
        except Exception:
            logger.exception("Couldn't set test cookie", extra={'request': request})
        return super(SetTestCookieMixin, self).get(request, *args, **kwargs)



# coding: utf-8
"""Utilities related to logging, log records, etc.

Source: lookup_www.common.log (with some modifications).

"""
from __future__ import absolute_import, print_function, unicode_literals


def not_http_404(log_record):
    """Check if ``log_record``'s ``status_code`` is not 404.

    :param log_record: log record
    :type log_record: :class:`logging.LogRecord`
    :return: False if ``log_record``'s ``status_code`` is 404

    """
    # LogRecord objects created by logger `django.request` have attribute `status_code`
    try:
        if log_record.status_code == 404:
            return False
    except Exception:
        pass
    return True



# coding: utf-8
"""Some useful Django form fields.

"""
from __future__ import absolute_import, print_function, unicode_literals

import django.forms


class LowerCaseEmailField(django.forms.EmailField):

    def __init__(self, max_length=None, min_length=None, lower_case=False, *args, **kwargs):
        self.lower_case = lower_case
        super(LowerCaseEmailField, self).__init__(max_length, min_length, *args, **kwargs)

    def clean(self, value):
        value = self.to_python(value).strip()
        if self.lower_case:
            value = value.lower()
        return super(LowerCaseEmailField, self).clean(value)



# coding: utf-8
"""Email-related utilities to create, render and send messages.

Source: lookup_www.common.utils.mail (with some edition).

"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

import django.core.mail
from django.template.loader import render_to_string


logger = logging.getLogger(__name__)


class PlainEmail(object):

    """Friendly wrapper over :class:`django.core.mail.EmailMessage`

    .. note::
        The only public attribute is :attr:`connection` because it can be
        changed safely after instantiation.

    """

    django_mail_class = django.core.mail.EmailMessage

    def __init__(
            self, subject='', body='', from_email=None, to=None, cc=None,
            bcc=None, reply_to=None, connection=None, attachments=None,
            headers=None):

        self._subject = subject
        self._body = body

        self._from_email = from_email
        self._to = to
        self._cc = cc
        self._bcc = bcc

        self.connection = connection
        self._attachments = attachments

        headers = set_header_reply_to(headers, reply_to)
        self._headers = headers

    def create_message(self):
        """Create an email message.

        :return: email message object
        :rtype: :attr:`django_mail_class`

        """
        return self.django_mail_class(
            subject=self._subject,
            body=self._body,
            from_email=self._from_email,
            to=self._to,
            bcc=self._bcc,
            connection=self.connection,
            attachments=self._attachments,
            headers=self._headers,
            cc=self._cc)

    def send(self):
        """Create the email message and send it.

        :return: the number of email messages sent (return value of :meth:`EmailMessage.send`)
        :rtype: int

        """
        message = self.create_message()
        return message.send()


class HTMLEmail(PlainEmail):

    """Customization of :class:`PlainEmail` to send a multipart HTML email."""

    django_mail_class = django.core.mail.EmailMultiAlternatives
    _EMAIL_ALTERNATIVE_MIME_TYPE = 'text/html'

    def __init__(self, html_body, *args, **kwargs):
        super(HTMLEmail, self).__init__(*args, **kwargs)
        self._html_body = html_body

    def create_message(self):
        message = super(HTMLEmail, self).create_message()
        message.attach_alternative(self._html_body, self._EMAIL_ALTERNATIVE_MIME_TYPE)
        return message


def send_mail2(
        subject='', body='', from_email=None, to=None, cc=None, bcc=None,
        reply_to=None, fail_silently=False, auth_user=None,
        auth_password=None, connection=None, attachments=None, headers=None):
    """Combination of :func:`django.core.mail.send_mail` and
    :class:`django.core.mail.EmailMessage`.

    The original ``send_mail`` was too limited and kind of deprecated ('The API
    for this method is frozen'). This function augments it.

    These are the parameters differences:
    * ``message`` renamed to ``body`` (compliant with ``EmailMessage``)
    * ``recipient_list`` renamed to ``to`` (compliant with ``EmailMessage``)
    * added ``cc``, ``bcc``, ``reply_to``, ``attachments`` and ``headers``
    * set default values for ``subject``, ``message``/``body``, ``from_email``,
        ``recipient_list``/``to``

    Check parameters definitions at :class:`django.core.mail.EmailMessage`
    and :func:`django.core.mail.get_connection`.

    :return: the number of email messages sent (return value of
        :meth:`EmailMessage.send`)

    .. warning::
        All passed email addresses must be validated beforehand. For example,
        all these values **WILL** result in a message being sent
        (i.e. return value will be ``> 0``:
        ``['', ' ', 'x', 'x@x']``
        Other values may raise exceptions e.g. ValueError and
        its subclass BadHeaderError.

    .. warning::
        Do not confuse ``from_email`` with ``auth_user``.

    """
    # 1. basic validation of given addresses lists/tuples
    # 2. get the connection
    # 3. set header 'Reply-To' (if given)
    # 4. create the EmailMessage object
    # 5. send it

    if to is not None:
        validate_addresses(to)
    if cc is not None:
        validate_addresses(cc)
    if bcc is not None:
        validate_addresses(bcc)

    if connection is None:
        connection = django.core.mail.get_connection(
            username=auth_user,
            password=auth_password,
            fail_silently=fail_silently)

    headers = set_header_reply_to(headers, reply_to)
    message = django.core.mail.EmailMessage(
        subject, body, from_email, to, bcc, connection, attachments, headers, cc)

    return message.send()


def send_templated_mail(
        context, subject_template_name, body_template_name, html_body_template_name=None,
        **kwargs):
    """Send mail by rendering subject and body templates using ``context``.

    The message will be a plain text or HTML multipart email depending on
    whether ``html_body_template_name`` is None or not.

    ``kwargs`` is passed directly to :class:`PlainEmail`, allowing full
    customization of the email creation (e.g. to, cc, bcc, headers, attachments)
    and send (i.e. connection object) process.

    :param context: context data used to render the templates
    :type context: :class:`dict`
    :param subject_template_name: the name of the subject template
    :type subject_template_name: string
    :param body_template_name: the name of the plain-text body template
    :type body_template_name: string
    :param html_body_template_name: if a multipart HTML email is desired, the
        name of the HTML body template; None otherwise
    :type html_body_template_name: string or None
    :return: True if at least one message is sent successfully; False otherwise
    :rtype: bool

    .. warning::
        All passed email addresses **must** be validated beforehand.

    .. warning::
        ``to``, ``cc`` and ``bcc`` arguments must be **iterables** of strings,
        not plain strings, no matter if their length is 1.

    """
    #noinspection PyBroadException
    try:
        subject = render_to_string(subject_template_name, context)
        subject = ''.join(subject.splitlines())
        body = render_to_string(body_template_name, context)
        if html_body_template_name is None:
            html_body = None
        else:
            html_body = render_to_string(html_body_template_name, context)
    except Exception:
        logger.exception(
            "Failed to render body or subject.\nTemplate names: (%s, %s, %s)\nContext: %s" %
            (subject_template_name, body_template_name, html_body_template_name, context))
        return False

    #noinspection PyBroadException
    try:
        if html_body is None:
            message = PlainEmail(subject=subject, body=body, **kwargs)
        else:
            message = HTMLEmail(html_body, subject=subject, body=body, **kwargs)

        result = message.send()
        if not result >= 1:
            logger.error("No mail was sent.\nsubject: %s\nbody: %s\nkwargs: %s" %
                         (subject, body, kwargs))
            return False
    except Exception:
        logger.exception("send_templated_mail failed.\nsubject: %s\nbody: %s\nkwargs: %s" %
                         (subject, body, kwargs))
        return False
    return True


def set_header_reply_to(headers, reply_to):
    """Set ``reply_to`` as email header ``'Reply-To'``.

    :param headers: email headers
    :type headers: dict or None
    :param reply_to: email address to set as header ``'Reply-To'``
    :type reply_to: basestring or None
    :return: updated headers, if ``reply_to`` is not None
    :rtype: dict, if ``reply_to`` is not None; unknown otherwise

    >>> set_header_reply_to(None, 'a@a.com')
    {'Reply-To': 'a@a.com'}
    >>> set_header_reply_to({}, 'a@a.com')
    {'Reply-To': 'a@a.com'}
    >>> set_header_reply_to({'Reply-To': 'a@a.com'}, 'b@b.com')
    {'Reply-To': 'b@b.com'}
    >>> set_header_reply_to(None, None)
    None
    >>> set_header_reply_to({}, None)
    {}

    """
    if reply_to is not None:
        if headers is None:
            headers = {'Reply-To': reply_to}
        else:
            headers['Reply-To'] = reply_to
    return headers


def validate_addresses(addresses):
    """Check that ``addresses`` can be joined in a single string.

    :param addresses: email addresses
    :rtype: list or tuple
    :raises: ValueError if ``addresses`` couldn't be joined

    """
    addresses = list(addresses)
    try:
        ', '.join(addresses)
    except TypeError:
        raise ValueError("Email addresses couldn't be joined in a single string: %s\n"
                         "Hint: probably one of them is not a string" % addresses)



# coding: utf-8
"""Source: lookup_www.lu_base.tests (but just a subset).

"""
from __future__ import absolute_import, print_function, unicode_literals

import unittest


class Functions(unittest.TestCase):

    def test_normalize_email(self):
        from lu_dj_utils.normalization import normalize_email

        self.assertEqual('a@a.com', normalize_email('A@A.COM'))
        self.assertEqual('a@a.com', normalize_email(' a@a.com '))
        self.assertEqual('a@a.com', normalize_email('\t  a@a.com\n'))

    def test_normalize_name(self):
        from lu_dj_utils.normalization import normalize_name

        self.assertEqual(normalize_name('oneTwoThree'), 'one_two_three')
        self.assertEqual(normalize_name('UnoAUno'), 'uno_a_uno')
        self.assertEqual(normalize_name('UnoAUNo'), 'uno_au_no')



# coding: utf-8
from __future__ import absolute_import, print_function, unicode_literals

import unittest

from django.utils import six


class FunctionsTest(unittest.TestCase):

    def test_hex_sha1(self):
        from lu_dj_utils.hex import hex_sha1, SHA1_RE

        self.assertEqual(
            hex_sha1(''.encode('utf-8'),
                     ''.encode('utf-8')),
            'da39a3ee5e6b4b0d3255bfef95601890afd80709')
        self.assertEqual(
            hex_sha1('chao1'.encode('utf-8'),
                     'hola'.encode('utf-8')),
            'bf01a6b75b943b6be4b0d5cce120d5cee9559217')
        self.assertEqual(
            hex_sha1('hola'.encode('utf-8'),
                     'chao1'.encode('utf-8')),
            '61807d12b62246bbe5f46955746b9a4ffbc6fb0d')
        self.assertEqual(
            hex_sha1('hola'.encode('utf-8'),
                     'chao1'.encode('utf-8')),
            '61807d12b62246bbe5f46955746b9a4ffbc6fb0d')

        # test results match the corresponding regex
        for i in range(0, 10000):
            value = hex_sha1(''.encode('utf-8'), six.text_type(i).encode('utf-8'))
            self.assertIsNotNone(SHA1_RE.match(value))

    def test_random_hex_32(self):
        # values random enough such that there are no repetitions in 10,000 calls

        from lu_dj_utils.hex import random_hex_32, HEX_32_RE

        keys = {}
        size_ = 10000
        for i in range(0, size_):
            random_key = random_hex_32()
            keys[random_key] = None
            # test results match the corresponding regex
            self.assertIsNotNone(HEX_32_RE.match(random_key))
        # if there were any repeated keys then len(keys) < size_
        self.assertEquals(size_, len(keys))

    def test_random_hex_6(self):
        # values random enough such that there are no repetitions in 100 calls
        # (with 1,000 it fails frequently enough to be annoying)

        from lu_dj_utils.hex import random_hex_6, HEX_6_RE

        keys = {}
        size_ = 100
        for i in range(0, size_):
            random_key = random_hex_6()
            keys[random_key] = None
            # test results match the corresponding regex
            self.assertIsNotNone(HEX_6_RE.match(random_key))
        # if there were any repeated keys then len(keys) < size_
        self.assertEquals(size_, len(keys))



# coding: utf-8
"""Functions and constants about hex values, UUID, hashes and random values.

Source: lookup_www.common.utils.hash

"""
from __future__ import absolute_import, print_function, unicode_literals

import hashlib
import re
import uuid


HEX_6_RE = re.compile('^[a-f0-9]{6}$')  # 6 hex digits string
HEX_32_RE = re.compile('^[a-f0-9]{32}$')  # 32 hex digits string
HEX_40_RE = re.compile('^[a-f0-9]{40}$')  # 40 hex digits string
SHA1_RE = HEX_40_RE
UUID4_RE = HEX_32_RE


def hex_sha1(salt, value):
    """Return a 40-character string containing only hexadecimal digits based on
    the string resulting from the concatenation of strings `salt` and `value`.
    These can be regular or unicode strings but non-ascii characters can not be
    used.

    Equivalent to hashlib.sha1(salt + value).hexdigest()

    :param salt:
    :type salt: six.binary_type
    :param value:
    :type value: six.binary_type

    """
    m = hashlib.sha1()
    m.update(salt)
    m.update(value)
    return m.hexdigest()


def random_hex_32():
    """Return a 32-character string containing only hexadecimal digits based on
    a random generated version-4 UUID.

    """
    return uuid.uuid4().hex


def random_hex_6():
    """Return a string of 6 hexadecimal digits based on :func:`random_hex_32`.

    """
    return random_hex_32()[:6]



# coding: utf-8
"""Source: lookup_www.lu_base.utils (but just an edited subset).

"""
from __future__ import absolute_import, print_function, unicode_literals

import re


def normalize_email(email):
    """Lowercase ``email`` and remove its leading and trailing whitespace.

    >>> normalize_email('A@A.COM')
    'a@a.com'
    >>> normalize_email('\t  a@a.com\n')
    'a@a.com'

    """
    return email.strip().lower()


def normalize_name(name):
    """Convert camel-case style names into underscore separated words.

    >>> normalize_name('oneTwoThree')
    'one_two_three'
    >>> normalize_name('FourFiveSix')
    'four_five_six'

    Source: `django.contrib.formtools.wizard.views.normalize_name`

    """
    new = re.sub('(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))', '_\\1', name)
    return new.lower().strip('_')



# coding: utf-8
"""Utilities for creating test cases.

Source: lookup_www.common.utils.test.

"""
from __future__ import absolute_import, print_function, unicode_literals

from mock import NonCallableMock

import django.test


class TestServerClient(django.test.Client):

    """Overrides constructor kwarg ``SERVER_NAME``.

    Some unit tests require that the host of the HTTP requests returned by
    the test client is a valid domain, and not the default ``testserver``.
    By default this class uses ``testclient.com``.

    This source helped: http://stackoverflow.com/questions/6291120/

    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('SERVER_NAME', 'testclient.com')
        super(TestServerClient, self).__init__(*args, **kwargs)


class MonkeyPatchedRF(django.test.RequestFactory):

    """Subclass that adds attributes ``session`` and ``user`` to requests.

    If the ``SessionMiddleware`` is enabled, "real" requests
    will have an attribute ``user``.

    If the ``AuthenticationMiddleware`` is enabled, "real" requests
    will have an attribute ``session``.

    By overriding method :meth:`request`, both attributes are added
    by default, although that can be changed with constructor kwargs
    ``add_session`` and ``add_user``.

    .. warning::
       The added attributes aren't "real", they are just mock objects
       (:class:`mock.NonCallableMock` specifically).

    """

    def __init__(self, add_session=True, add_user=True, **kwargs):
        self.add_session = add_session
        self.add_user = add_user
        super(MonkeyPatchedRF, self).__init__(**kwargs)

    def request(self, **request):
        req = super(MonkeyPatchedRF, self).request(**request)

        if self.add_session:
            req.session = NonCallableMock(name='request session')
            req.session.session_key = 'abc123'

        if self.add_user:
            req.user = NonCallableMock(name='request user')

        return req


def create_request(path, method='get', data=None, **extra):
    """Return a HTTP request for ``path`` and ``data`` using ``method``.

    This function is just a shortcut to avoid instantiation of
    :class:`MonkeyPatchedRF`. If you need many requests, it's better to
    create a request factory object and use it to create each one.

    :param path:
    :param method: HTTP method of the request, it can be any of the supported
        by :class:`django.test.RequestFactory` e.g. ``get``, ``post``,
        ``head``, ``options``, ``put`` or ``delete``
    :return: HTTP request for ``path`` and ``data`` using ``method``
    :rtype: :class:`django.core.handlers.wsgi.WSGIRequest`

    """
    method = method.lower()  # methods are in lower case in the RF class
    data = data or {}
    rf = MonkeyPatchedRF()

    func = getattr(rf, method)
    return func(path, data=data, **extra)


def setup_view(view, request, *args, **kwargs):
    """Mimic as_view() returned callable, but returns a view instance.

    :type args: the same you would pass to
        :func:`django.core.urlresolvers.reverse`
    :type kwargs: the same you would pass to
        :func:`django.core.urlresolvers.reverse`

    Source: http://tech.novapost.fr/django-unit-test-your-views-en.html

    """
    view.request = request
    view.args = args
    view.kwargs = kwargs
    return view



# coding: utf-8
"""Source: lookup_www.common.models (but just a subset)

"""
from __future__ import absolute_import, print_function, unicode_literals


###############################################################################
# MIXINS
###############################################################################


class ValidateModelMixin(object):

    """Make :meth:`save` call :meth:`full_clean`.

    .. warning:
        This should be the left-most mixin/super-class of a model.

    Do you think Django models ``save`` method will validate all fields
    (i.e. call ``full_clean``) before saving or any time at all? Wrong!

    I discovered this awful truth when I couldn't understand why
    a model object with an email field (without `blank=True`) could be
    saved with an empty string as email address.

    More info:

    * "Why doesn't django's model.save() call full clean?"
        http://stackoverflow.com/questions/4441539/
    * "Model docs imply that ModelForm will call Model.full_clean(),
        but it won't."
        https://code.djangoproject.com/ticket/13100

    """

    def save(self, *args, **kwargs):
        """Call :meth:`full_clean` before saving."""
        self.full_clean()
        super(ValidateModelMixin, self).save(*args, **kwargs)



# coding: utf-8
from __future__ import absolute_import, print_function, unicode_literals

from mock import MagicMock, Mock, NonCallableMock, NonCallableMagicMock, patch

from django.http import QueryDict
from django.test import RequestFactory, SimpleTestCase, TestCase
from django.test.utils import override_settings
from django.utils.six.moves.urllib.parse import (
    quote, quote_plus, unquote, unquote_plus, urlparse, urlunparse,
    urlsplit, urlunsplit,
)


class BuildQuerystring(SimpleTestCase):  # TODO: change to unittest.TestCase?

    def setUp(self):
        mock_param = NonCallableMock()
        self._dict = {'next': '/a&b/', 'x': mock_param}
        self._query_dict = QueryDict('', mutable=True)
        self._query_dict.update(self._dict)

        part_a = 'next=%2Fa%26b%2F'
        part_b = 'x=' + quote_plus(str(mock_param))
        self.expected = (part_a + '&' + part_b, part_b + '&' + part_a)

    def test_with_dict(self):
        from lu_dj_utils.urls import build_querystring

        result = build_querystring(self._dict)
        self.assertTrue(
            result == self.expected[0] or result == self.expected[1])

    def test_with_query_dict(self):
        from lu_dj_utils.urls import build_querystring

        result = build_querystring(self._query_dict)
        self.assertTrue(
            result == self.expected[0] or result == self.expected[1])


class Functions(TestCase):  # TODO: change to SimpleTestCase or unittest.TestCase?

    @override_settings(SITE_ID=10)
    def test_build_full_url(self):
        from django.contrib.sites.models import Site
        from lu_dj_utils.urls import build_full_url

        Site.objects.create(pk=10, domain='mycooldomain.com')
        site2 = Site.objects.create(domain='www.domain2.com')

        obj = Mock()
        obj.get_absolute_url = Mock(return_value='/asdfkas/asldfk/12/5')

        url = build_full_url(obj)
        self.assertEqual(url, 'http://mycooldomain.com/asdfkas/asldfk/12/5')
        url = build_full_url(obj, site2)
        self.assertEqual(url, 'http://www.domain2.com/asdfkas/asldfk/12/5')
        url = build_full_url(obj, protocol='sftp')
        self.assertEqual(url, 'sftp://mycooldomain.com/asdfkas/asldfk/12/5')

    @override_settings(SITE_ID=10)
    def test_get_full_url(self):
        from django.contrib.sites.models import Site
        from lu_dj_utils.urls import get_full_url

        Site.objects.create(pk=10, domain='mycooldomain.com')
        site2 = Site.objects.create(domain='www.domain2.com')

        absolute_url = '/asdfkas/asldfk/12/5'

        url = get_full_url(absolute_url)
        self.assertEqual(url, 'http://mycooldomain.com/asdfkas/asldfk/12/5')
        url = get_full_url(absolute_url, site2)
        self.assertEqual(url, 'http://www.domain2.com/asdfkas/asldfk/12/5')
        url = get_full_url(absolute_url, protocol='sftp')
        self.assertEqual(url, 'sftp://mycooldomain.com/asdfkas/asldfk/12/5')



#!/usr/bin/env python
from __future__ import absolute_import, print_function, unicode_literals

from setuptools import find_packages, setup

import lu_dj_utils


with open('README.rst') as f:
    readme = f.read()

packages = find_packages()

classifiers = (
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    # the FSF refers to it as "Modified BSD License". Other names include
    # "New BSD", "revised BSD", "BSD-3", or "3-clause BSD"
    'License :: OSI Approved :: BSD License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Framework :: Django',
)

setup(
    name="lu-dj-utils",
    version=lu_dj_utils.__version__,
    description="LookUP.cl's open source utilities for use in Django projects",
    long_description=readme,
    author='German Larrain',
    author_email='glarrain@users.noreply.github.com',
    url='https://github.com/lookup/lu-dj-utils',
    packages=packages,
    install_requires=['Django>=1.6'],
    license='3-clause BSD',  # TODO: verify name is correct
    zip_safe=False,
    classifiers=classifiers,

    tests_require=[
        'Django>=1.6',
        'mock>=1.0.1',
    ],
    test_suite='runtests.runtests',
)



# coding: utf-8
"""Some useful Django model fields.

Source: lookup_www.common.fields (but just a subset).

"""
from __future__ import absolute_import, print_function, unicode_literals

import django.db.models
import django.forms

import lu_dj_utils.hex


class Hex32Field(django.db.models.CharField):

    """Hex digits string of length 32, very practical for hash keys.

    Default value is calculated by :func:`lu_dj_utils.hex.random_hex_32` (UUID version 4).
    Although it's very unlikely that it returns a repeated value,
    uniqueness is enforced at database level as precautionary measure.

    """

    description = "32-hex-digits string"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = False
        kwargs['default'] = lu_dj_utils.hex.random_hex_32
        kwargs['max_length'] = 32
        kwargs['unique'] = True
        kwargs.setdefault('editable', False)
        super(Hex32Field, self).__init__(*args, **kwargs)


class Hex6Field(django.db.models.CharField):

    """Hex digits string of length 6.

    Default value is calculated by :func:`lu_dj_utils.hex.random_hex_6`
    (substring of UUID version 4).
    Uniqueness is enforced at database level as precautionary measure.

    """

    description = "6-hex-digits string"

    def __init__(self, *args, **kwargs):
        kwargs['blank'] = False
        kwargs['default'] = lu_dj_utils.hex.random_hex_6
        kwargs['max_length'] = 6
        kwargs['unique'] = True
        kwargs.setdefault('editable', False)
        super(Hex6Field, self).__init__(*args, **kwargs)


# Tell South about the custom field. Since it's essentially an IntegerField (as
# South and DB are concerned), the definitions are as simple as they can be.
# Read:
# * http://south.readthedocs.org/en/latest/customfields.html#extending-introspection
# * http://south.readthedocs.org/en/latest/tutorial/part4.html#simple-inheritance
from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^common\.fields\.Hex32Field"])
add_introspection_rules([], ["^common\.fields\.Hex6Field"])



# coding: utf-8
"""URL and HTTP related utilities.

Source: lookup_www.common.utils.urls (a subset with some edition).

"""
from __future__ import absolute_import, print_function, unicode_literals

from django.contrib.sites.shortcuts import get_current_site
from django.core.urlresolvers import reverse
from django.http import QueryDict
from django.utils.six.moves.urllib.parse import (
    quote, quote_plus, unquote, unquote_plus, urlparse, urlunparse,
    urlsplit, urlunsplit,
)


def build_querystring(query_dict):
    """Return a query string using ``query_dict`` contents.

    :param query_dict: contents of the query string
    :type query_dict: dict or :class:`django.http.QueryDict`
    :return: query string
    :rtype: str

    .. note::
        Note that ``query_dict`` can be a :class:`django.http.QueryDict`,
        which is a multi-valued dictionary.

    >>> q = {'next': '/a&b/'}
    >>> build_querystring(q)
    'next=%2Fa%26b%2F'

    >>> q = QueryDict('', mutable=True)
    >>> q['next'] = '/a&b/'
    >>> build_querystring(q)
    'next=%2Fa%26b%2F'

    """
    if not isinstance(query_dict, QueryDict):
        q = QueryDict('', mutable=True)
        q.update(query_dict)
        query_dict = q
    return query_dict.urlencode()


def cut_url_querystring(url):
    """Cut the querystring from ``url``, if present.

    :param url: str
    :type url: str
    :return: ``url`` without its querystring (if present)
    :rtype: str

    >>> cut_url_querystring('http://amazon.com/user/login/?qs')
    'http://amazon.com/user/login/'
    >>> cut_url_querystring('amazon.com/user/login/?qs')
    'amazon.com/user/login/'
    >>> cut_url_querystring('www.amazon.com/user/login/?qs')
    'www.amazon.com/user/login/'
    >>> cut_url_querystring('http://localhost:8081/user/login/?ref=/')
    'http://localhost:8081/user/login/'
    >>> cut_url_querystring('localhost/user/login/?ref=/')
    'localhost/user/login/'

    """
    new_url = ''
    temp = urlsplit(url)
    if temp.scheme:
        new_url += temp.scheme + '://'
    return new_url + temp.netloc + temp.path


def reverse_with_querystring(url_name, query_dict, *args, **kwargs):
    """Return a reversed URL plus a querystring.

    The full URL is composed of the URL resolved using ``url_name``, ``args``
    and ``kwargs`` plus the query string built with ``query_dict``.

    :param url_name: URL pattern name to be reversed/resolved
    :type url_name: str
    :param query_dict: contents of the query string
    :type query_dict: dict or :class:`django.http.QueryDict`
    :param args: passed to :func:`django.core.urlresolvers.reverse`
    :param kwargs: passed to :func:`django.core.urlresolvers.reverse`
    :return: full URL e.g. '/tmovil/planes/seleccion-comuna/?x=y&foo=bar&foo='
    :rtype: str

    """
    querystring = build_querystring(query_dict)
    reversed_url = reverse(url_name, *args, **kwargs)
    return reversed_url + '?' + querystring


def build_full_url(obj, site=None, protocol='http'):
    """Build a full URL considering current site and ``obj``'s absolute URL.

    :param obj: must have a method named ``get_absolute_url``
    :type obj: :class:`django.db.models.Model`
    :param site: the site to use for the URL; if None, the current site will
        be used
    :type site: :class:`django.contrib.sites.models.Site` or None
    :param protocol: e.g. ftp, https
    :type protocol: string
    :return: full URL to access ``obj``
    :rtype: string

    >> build_full_url(Telefono.objects.get(slug='alcatel-282')
    http://www.lookup.cl/tmovil/telefonos/alcatel-282/

    """
    if site is None:
        site = get_current_site(None)
    # noinspection PyUnresolvedReferences
    return ''.join([protocol, '://', str(site), obj.get_absolute_url()])


def get_full_url(absolute_url, site=None, protocol='http'):
    """Convert ``absolute_url`` to a full URL considering current site.

    :param absolute_url: URL to convert
    :type absolute_url: str
    :param site: the site to use for the URL; if None, the current site will
        be used
    :type site: :class:`django.contrib.sites.models.Site` or None
    :param protocol: e.g. ftp, https
    :type protocol: string
    :return: full URL
    :rtype: string

    .. warning::
        ``absolute_url`` must begin with a slash character (``/``).

    >>> get_full_url('/x/abc/?ksdfd')
    http://www.lookup.cl/x/abc/?ksdfd

    """
    if site is None:
        site = get_current_site(None)
    return ''.join([protocol, '://', str(site), absolute_url])
